//******************************************************************
//                                                                 
//  TestClient.java                                               
//  Copyright 2014 PSI AG. All rights reserved.              
//  PSI PROPRIETARY/CONFIDENTIAL. Use is subject to license terms
//                                                                 
// ******************************************************************

package test.test.wildfly;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.google.common.base.Strings;

/**
 * TODO Describe the type (The first line - until the first dot - will interpret as the brief description).
 * 
 * @see add further links
 * @author created: mkozlowicz on 20 cze 2014 11:34:19
 * @author last change: $Author: $ on $Date: $
 * @version $Revision: $
 */
public class TestClient
{

    /**
     * Constructs the JNDI coordinates for the given EJB.
     * <p>
     * This method uses the EJB Client API coordinates (starting with <code>ejb:</code>) and doesn't do any
     * lookup - it merely provides you with the JNDI coordinates.
     * </p>
     * <p>
     * The format of the EJB Client API is as follows: <br/>
     * <code>ejb:[appName]/[moduleName]/[beanName][!interfaceName][?stateful]</code>
     * </p>
     * 
     * @param aAppName
     *            application name
     * @param aModuleName
     *            module name
     * @param aBeanName
     *            bean name
     * @param aInterfaceName
     *            interface name
     * @param aIsStateful
     *            is it the SFSB?
     * @return JNDI coordinates
     */
    static String getJNDICoordinates( String aAppName, String aModuleName, String aBeanName,
        String aInterfaceName, boolean aIsStateful )
    {
        String result = "ejb:" + aAppName + "/" + aModuleName + "/" + aBeanName;

        if( !Strings.isNullOrEmpty( aInterfaceName ) )
        {
            result += "!" + aInterfaceName;
        }

        if( aIsStateful )
        {
            result += "?stateful";
        }

        return result;
    }

    @SuppressWarnings( "unchecked" )
    public static < T >T lookup( Class< ? > aInterfaceName, String appName, String moduleName,
        String aBeanName ) throws NamingException
    {
        final String jndi =
            getJNDICoordinates( appName, moduleName, aBeanName, aInterfaceName.getName(), false );
        Object ob = simpleLookup( jndi );
        return (T)ob;
    }

    public static Object simpleLookup( final String jndiName ) throws NamingException
    {
        Properties props = new Properties();
        props.put( Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming" );
        InitialContext context = new InitialContext( props );
        Object ob = context.lookup( jndiName );
        context.close();
        return ob;
    }

    @SuppressWarnings( "unchecked" )
    public static < T >T lookup( Class< ? > aInterface, String beanName ) throws NamingException
    {
        final String jndi =
            getJNDICoordinates( "server.dist", "server-0.1.0-SNAPSHOT", beanName, aInterface.getName(), false );
        Object ob = simpleLookup( jndi );
        return (T)ob;
    }

    public static void main( String[] args ) throws NamingException
    {
        TestRemote remote = lookup( TestRemote.class, "TestRemoteImpl" );
        System.out.println( remote.getVersion() );
    }
}
